#! /bin/sh -e

# Helpers
## Function to log messages
log_helper() {
    LOG_FILE=/tmp/printserver-install.log
    LOG_MSG=$1
    echo "$(date +"%y-%m-%d %T") - $LOG_MSG" >> $LOG_FILE
    echo $LOG_MSG
}

## Function to install packages in unattended mode
apt_helper() {
    APT_CMD=$@
    DEBIAN_FRONTEND=noninteractive APT_LISTCHANGES_FRONTEND=none \
    sudo apt-get -fuy -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" \
    $APT_CMD
}

# Display context
echo "A log file will be created as /tmp/printserver-install.log"

# Updating system source repos
msg="Updating system source repos"
log_helper "$msg ... "
sudo apt update
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Installing mosquitto
msg="Installing mosquitto"
log_helper "$msg ... "
sudo snap install mosquitto 
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Installing unzip
msg="Installing unzip"
log_helper "$msg ... "
apt_helper install unzip
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Getting latest application version
msg="Getting latest application version"
log_helper "$msg ... "
app_last_version=$(wget -qO- https://www.waboo.ma/lacaissema/printserver/latest)
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "Latest application version found $app_last_version."

# Downloading latest application version
msg="Downloading latest application version"
log_helper "$msg ... "
cd
wget "https://www.waboo.ma/lacaissema/printserver/$app_last_version"
if [ "$?" -ne 0 ]
then
    echo "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Unziping the archive
msg="Unziping the archive"
log_helper "$msg ... "
unzip $app_last_version
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Making required folders
msg="Making required folders"
log_helper "$msg ... "
mkdir -p printserver/pyenv
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"
cd printserver/

# Installing virtual environment module
msg="Installing virtual environment module"
log_helper "$msg ... "
sudo apt_helper install python3-venv
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Creating and activating a new virtual environment
msg="Creating and activating a new virtual environment"
log_helper "$msg ... "
python3 -m venv pyenv
. pyenv/bin/activate
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Installing required python libraries
msg="Installing Flask==1.1.2"
log_helper "$msg ... "
pip3 install Flask==1.1.2
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

msg="Installing gunicorn==20.1.0"
log_helper "$msg ... "
pip3 install gunicorn==20.1.0
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

msg="Installing python-escpos==2.2.0"
log_helper "$msg ... "
pip3 install python-escpos==2.2.0
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

msg="Installing paho-mqtt==1.5.1"
log_helper "$msg ... "
pip3 install paho-mqtt==1.5.1
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Setting up printserverservices as a System Service
msg="Setting up printserverservices as a System Service"
log_helper "$msg ... "
sudo cp ~/printserver/deployment/printserverservices.service /etc/systemd/system/
sudo systemctl daemon-reload
sudo systemctl enable printserverservices
sudo systemctl start printserverservices
sudo systemctl is-active --quiet printserverservices
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Setting up printserverapi as a System Service
msg="Setting up printserverapi as a System Service"
log_helper "$msg ... "
sudo cp ~/printserver/deployment/printserverapi.service /etc/systemd/system/
sudo systemctl daemon-reload
sudo systemctl enable printserverapi
sudo systemctl start printserverapi
sudo systemctl is-active --quiet printserverapi
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Changing system timezone to Africa/Casablanca
msg="Changing system timezone to Africa/Casablanca"
log_helper "$msg ... "
sudo timedatectl set-timezone Africa/Casablanca
if [ "$?" -ne 0 ]
then
    log_helper "$msg failed, exiting."
    exit
fi
log_helper "... done"

# Setup finished successfully
log_helper "Setup finished successfully!"